/*
  Nanonyam
  Copyright (C) 2020 Nanoslavic
  www.nanoslavic.ru/nanonyam
  www.nanoslavic.com/nanonyam
  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
*/

#include <stdio.h>
#include "nanonyam.h"
#include "nn_system_lib.h"
#include "nn_file_lib.h"

//     (id) -    .    10   (id=0...255).
//          GetLastError().
//      ,       FileSetCurrentDir.
//    ,     NanoNyam.exe.
//!       ,   "\"   "\\",     .
//: FileOpen(0,0,"C:\\Program files\\program\\text.txt");
//      .


//    (filename)   (mode).    ,    .
//    (id)   ,      .  GetLastError()>0,  .
//(mode=0) -      ( ,  )
//(mode=1) -      ( ,  )
//(mode=2) -        ( ,  )
//(mode=3) -      ( )
//(mode=4) -      ( )
//(mode=5) -        ( )
void file_Open(uint8_t id,uint8_t mode,const char* filename){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+1)))=mode;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=(uint16_t)filename;
	sys_ExecuteByCodes(7,40);
}

//  (id).  GetLastError()>0,  .
void file_Close(uint8_t id){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	sys_ExecuteByCodes(7,41);
}

//        ,     .
// FileFlush      (id).  GetLastError()>0,  .
void file_Flush(uint8_t id){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	sys_ExecuteByCodes(7,42);
}

//    (filename).    .  GetLastError()>0,  .
void file_Delete(const char* filename){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)filename;
	sys_ExecuteByCodes(7,43);
}

//    (dirname).  GetLastError()>0,  .
void file_CreateDir(const char* dirname){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)dirname;
	sys_ExecuteByCodes(7,44);
}

//    (dirname).  GetLastError()>0,  .
void file_DeleteDir(const char* dirname){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)dirname;
	sys_ExecuteByCodes(7,60);
}


// 1,     (filename) .
uint8_t file_Exist(const char* filename){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)filename;
	return sys_ExecuteByCodesU8(7,45);
}

//    .  GetLastError()>0,  .
void file_ToHome(uint8_t id){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	sys_ExecuteByCodes(7,46);
}

//    .  GetLastError()>0,  .
void file_ToEnd(uint8_t id){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	sys_ExecuteByCodes(7,48);
}

//    (position).  GetLastError()>0,  .
void file_ToPos(uint8_t id,uint32_t position){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile uint32_t*)(START_SYSTEM_PARAM_ADDR+1)))=position;
	sys_ExecuteByCodes(7,47);
}

//        .      .
// GetLastError()>0,  .   ,  GetLastError()=0xFF.
uint8_t file_ReadU8(uint8_t id){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	return sys_ExecuteByCodesU8(7,49);
}
uint16_t file_ReadU16(uint8_t id){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	return sys_ExecuteByCodesU16(7,50);
}
uint32_t file_ReadU32(uint8_t id){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	return sys_ExecuteByCodesU32(7,51);
}
uint64_t file_ReadU64(uint8_t id){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	return sys_ExecuteByCodesU64(7,52);
}
float file_ReadFLT(uint8_t id){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	return sys_ExecuteByCodesFLT(7,51);
}

//        .      .
//    ,   ,  .  GetLastError()>0,  .
void file_WriteU8(uint8_t id,uint8_t data){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+1)))=data;
	sys_ExecuteByCodes(7,53);
}

void file_WriteU16(uint8_t id,uint16_t data){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+1)))=data;
	sys_ExecuteByCodes(7,54);
}

void file_WriteU32(uint8_t id,uint32_t data){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile uint32_t*)(START_SYSTEM_PARAM_ADDR+1)))=data;
	sys_ExecuteByCodes(7,55);
}

void file_WriteFLT(uint8_t id,float data){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile float*)(START_SYSTEM_PARAM_ADDR+1)))=data;
	sys_ExecuteByCodes(7,55);
}

void file_WriteU64(uint8_t id,uint64_t data){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile uint64_t*)(START_SYSTEM_PARAM_ADDR+1)))=data;
	sys_ExecuteByCodes(7,56);
}

// (num)       .     .
// GetLastError()>0,  .   ,  GetLastError()=0xFF.
void file_Read(uint8_t id,uint16_t num,uint8_t *data){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+1)))=num;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+3)))=(uint16_t)data;
	sys_ExecuteByCodes(7,57);
}

// (num)       .     .
//    ,   ,  .  GetLastError()>0,  .
void file_Write(uint8_t id,uint16_t num,uint8_t *data){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+1)))=num;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+3)))=(uint16_t)data;
	sys_ExecuteByCodes(7,58);
}

//      text.
void file_WriteText(uint8_t id,const char *text){
(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+1)))=(uint16_t)text;
	sys_ExecuteByCodes(7,100);
}

//      text      0x0D+0x0A.
void file_WriteTextLine(uint8_t id,const char *text){
(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+1)))=(uint16_t)text;
	sys_ExecuteByCodes(7,101);
}

//       (  0xA)   text.   .
//max -     data.
// GetLastError()>0,  .
uint16_t file_ReadTextLine(uint8_t id,uint16_t max,char* text){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+1)))=(uint16_t)text;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+3)))=max;
	return sys_ExecuteByCodesU16(7,98);
}

//      num   text.
// GetLastError()>0,  .
void file_ReadText(uint8_t id,uint16_t num,char* text){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+1)))=(uint16_t)text;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+3)))=num;
	sys_ExecuteByCodes(7,99);
}

//    (id).    .
uint32_t file_GetSize(uint8_t id){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=id;
	return sys_ExecuteByCodesU32(7,59);
}

//      (oldname)   (newname).    .  GetLastError()>0,  .
void file_Rename(const char* oldname,const char* newname){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)oldname;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=(uint16_t)newname;
	sys_ExecuteByCodes(7,61);
}

//  .      .  GetLastError()>0,  .
void file_SetCurrentDir(const char* dirname){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)dirname;
	sys_ExecuteByCodes(7,64);
}

//    (dirname).      .
//(max) -   (dirname).  GetLastError()>0,  .
void file_GetCurrentDir(uint16_t max,char* dirname){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)dirname;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=max;
	sys_ExecuteByCodes(7,62);
}

//    (dirname), (max) -   (dirname).  GetLastError()>0,  .
void file_GetTempDir(uint16_t max,char* dirname){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)dirname;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=max;
	sys_ExecuteByCodes(7,63);
}

// ,    ,  (dirname), (max) -   (dirname).  GetLastError()>0,  .
void file_GetProgramDir(uint16_t max,char* dirname){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)dirname;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=max;
	sys_ExecuteByCodes(7,69);
}

//    (name)   ".exe", (max) -   (name).  GetLastError()>0,  .
void file_GetProgramName(uint16_t max,char* name){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)name;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=max;
	sys_ExecuteByCodes(7,68);
}

// ,    ,  (dirname), (max) -   (dirname).  GetLastError()>0,  .
void file_GetCodeDir(uint16_t max,char* dirname){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)dirname;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=max;
	sys_ExecuteByCodes(7,102);
}

//   ,  (pathname), (max) -   (dirname).  GetLastError()>0,  .
void file_GetCodePath(uint16_t max,char* pathname){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)pathname;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=max;
	sys_ExecuteByCodes(7,103);
}