/*
  Nanonyam
  Copyright (C) 2020 Nanoslavic
  www.nanoslavic.ru/nanonyam
  www.nanoslavic.com/nanonyam
  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
*/ 

//Graphic functions

#ifndef __nanonyam_graphics_lib
#define __nanonyam_graphics_lib
//    
#define g_RGB(r,g,b)	   ((uint32_t)((uint32_t)(r)|((uint32_t)(g) << 8)|((uint32_t)(b) << 16)))
#define g_GetRed(rgb)	   ((uint8_t)(rgb))
#define g_GetGreen(rgb)	   ((uint8_t)(((uint16_t)(rgb)) >> 8))
#define g_GetBlue(rgb)	   ((uint8_t)((rgb) >> 16))


#define MAX_SCREEN_WIDTH		10010
#define MAX_SCREEN_HEIGHT		10010

//Basics
void g_SetScreenSize(int16_t sizex, int16_t sizey);
void g_Update(void);
void g_FillAll(void);
void g_ClearAll(void);
void g_CreatePage(uint16_t pageN,int16_t width,int16_t height,uint8_t colors);
void g_DeletePage(uint16_t pageN);
void g_SetPage(uint16_t n);
uint16_t g_GetPageNum(void);
int16_t g_GetPageWidth(uint16_t pageN);
int16_t g_GetPageHeight(uint16_t pageN);
void g_CopyPage(uint16_t pageN,int16_t xsource,int16_t ysource,int16_t x,int16_t y,int16_t width,int16_t height);
uint8_t g_GetPageStatus(uint16_t pageN);
void g_SetPen(int16_t size,uint32_t color);
void g_SetPenSize(int16_t size);
int16_t g_GetPenSize(void);
void g_SetPenColor(uint32_t color);
void g_SetPenRGB(uint8_t red,uint8_t green,uint8_t blue);
uint32_t g_GetPenColor(void);
void g_SetFillColor(uint32_t color);
void g_SetFillRGB(uint8_t red,uint8_t green,uint8_t blue);
uint32_t g_GetFillColor(void);
void g_SetBackColor(uint32_t color);
void g_SetBackRGB(uint8_t red,uint8_t green,uint8_t blue);
uint32_t g_GetBackColor(void);
uint8_t g_CheckRegionXY(int16_t x, int16_t y);
void g_SetDrawRegion(int16_t x1, int16_t y1,int16_t x2, int16_t y2);
void g_SetMaxDrawRegion(void);
void g_SetColorXY(int16_t x, int16_t y,uint32_t color);
uint32_t g_GetColorXY(int16_t x, int16_t y);
uint32_t g_MixColors(uint8_t alpha,uint32_t color1,uint32_t color2);
void g_FloodFill(int16_t x,int16_t y,uint8_t mode);
uint32_t g_RandomColor(void);

//Drawing
void g_DrawPixel(int16_t x, int16_t y);
void g_ClearPixel(int16_t x, int16_t y);
void g_InvertPixel(int16_t x, int16_t y);
void g_RandomPixel(int16_t x, int16_t y);
void g_DrawPoint(int16_t x, int16_t y);
void g_FillRect(int16_t x1, int16_t y1,int16_t x2, int16_t y2);
void g_ClearRect(int16_t x1, int16_t y1,int16_t x2, int16_t y2);
void g_InvertRect(int16_t x1, int16_t y1,int16_t x2, int16_t y2);
void g_FillRectRandom(int16_t x1, int16_t y1,int16_t x2, int16_t y2);
void g_DrawLineH(int16_t x,int16_t y,int16_t L);
void g_DrawLineV(int16_t x,int16_t y,int16_t L);
void g_DrawLine(int16_t x1, int16_t y1,int16_t x2, int16_t y2);
void g_DrawRectWH(int16_t x, int16_t y,int16_t width, int16_t height);
void g_DrawRect(int16_t x1, int16_t y1,int16_t x2, int16_t y2);
void g_DrawFillRect(int16_t x1, int16_t y1,int16_t x2, int16_t y2);
void g_DrawFillRectWH(int16_t x, int16_t y,int16_t width, int16_t height);
void g_DrawCircle(int16_t x,int16_t y,int16_t R);
void g_DrawFillCircle(int16_t x,int16_t y,int16_t R);
void g_DrawEllipse(int16_t x1, int16_t y1,int16_t x2, int16_t y2);
void g_DrawFillEllipse(int16_t x1, int16_t y1,int16_t x2, int16_t y2);
void g_DrawRoundRect(int16_t x1, int16_t y1,int16_t x2, int16_t y2, int16_t r);
void g_DrawFillRoundRect(int16_t x1, int16_t y1,int16_t x2, int16_t y2, int16_t r);
void g_DrawRoundRectWH(int16_t x, int16_t y,int16_t w, int16_t h, int16_t r);
void g_DrawFillRoundRectWH(int16_t x, int16_t y,int16_t w, int16_t h, int16_t r);

void g_MoveTo(int16_t x, int16_t y);
void g_LineTo(int16_t x, int16_t y);
void g_SetArcDirection(uint8_t dir);
void g_DrawArc(int16_t left,int16_t top,int16_t right,int16_t bottom,int16_t startX,int16_t startY,int16_t endX,int16_t endY);
void g_DrawArcTo(int16_t left,int16_t top,int16_t right,int16_t bottom,int16_t startX,int16_t startY,int16_t endX,int16_t endY);
void g_DrawChord(int16_t left,int16_t top,int16_t right,int16_t bottom,int16_t startX,int16_t startY,int16_t endX,int16_t endY);
void g_DrawPie(int16_t left,int16_t top,int16_t right,int16_t bottom,int16_t startX,int16_t startY,int16_t endX,int16_t endY);
void g_DrawAngleArc(int16_t x,int16_t y,int16_t radius,float startA,float sweep);
void g_DrawBezier(int16_t x0,int16_t y0,int16_t cx0,int16_t cy0,int16_t cx1,int16_t cy1,int16_t x1,int16_t y1);

//Text
void g_SetInternalFont(uint16_t font, int16_t pitch, uint16_t n);
void g_ConfigExternalFont(uint8_t font_id,uint16_t height, uint8_t bold, uint8_t italic, uint8_t underline,uint8_t strikeout,const char* name);
void g_SetExternalFont(uint8_t font_id);
void g_SetTextColor(uint32_t color);
void g_SetTextRGB(uint8_t red,uint8_t green,uint8_t blue);
uint32_t g_GetTextColor(void);
void g_GetTextRect(int16_t *width, int16_t *height,const char *text);
int16_t g_DrawText(int16_t x, int16_t y,const char *text);
void g_DrawTextCenterX(int16_t x0, int16_t x1, int16_t y,const char *text);

//Files
void g_GetSizeBMP(const char* filepath,int16_t* width,int16_t* height);
void g_DrawBMP(const char* filepath,int16_t x,int16_t y);
void g_CreatePageFromBMP(uint16_t pageN,const char* filepath);
void g_SavePageToBMP(uint16_t pageN,const char* filepath);
void g_SaveRectToBMP(const char* filepath,int16_t x0,int16_t y0,int16_t x1,int16_t y1);

#endif /*__nanonyam_graphics_lib*/
