/*
  Nanonyam
  Copyright (C) 2020 Nanoslavic
  www.nanoslavic.ru/nanonyam
  www.nanoslavic.com/nanonyam
  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
*/ 

#include <stdio.h>
#include "nanonyam.h"
#include "nn_system_lib.h"
#include "nn_rs232_lib.h"


// COM-   (portnum).  GetLastError()>0,  .
void rs232_Open(uint8_t portnum){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	sys_ExecuteByCodes(7,73);
}
// COM-   (portnum).  GetLastError()>0,  .
//        .
void rs232_Close(uint8_t portnum){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	sys_ExecuteByCodes(7,74);
}

//  COM-   (portnum).  GetLastError()>0,  .
//(baudrate) -    (, 19200); (databits) -    ( 8);
//(patity) -  ( 0); (stopbits) -  - ( 1).
void rs232_Config(uint8_t portnum,uint32_t baudrate,uint8_t databits,uint8_t patity,uint8_t stopbits){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	(*((volatile uint32_t*)(START_SYSTEM_PARAM_ADDR+1)))=baudrate;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+5)))=databits;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+6)))=patity;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+7)))=stopbits;
	sys_ExecuteByCodes(7,75);
}


//    (, , 1000)  COM-   (portnum).  GetLastError()>0,  .
void rs232_SetTimeOut(uint8_t portnum,uint32_t timeout){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	(*((volatile uint32_t*)(START_SYSTEM_PARAM_ADDR+1)))=timeout;
	sys_ExecuteByCodes(7,76);
}

// (num)    (data)  COM-   (portnum).  GetLastError()>0,  .
void rs232_Write(uint8_t portnum,uint16_t num,uint8_t *data){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+1)))=num;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+3)))=(uint16_t)data;
	sys_ExecuteByCodes(7,77);
}

// (num)    (data)  COM-   (portnum).  GetLastError()>0,  .
void rs232_Read(uint8_t portnum,uint16_t num,uint8_t *data){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+1)))=num;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+3)))=(uint16_t)data;
	sys_ExecuteByCodes(7,78);
}

// 1  (data)  COM-   (portnum).  GetLastError()>0,  .
void rs232_WriteByte(uint8_t portnum,uint8_t data){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+1)))=data;
	sys_ExecuteByCodes(7,79);
}
//   1   COM-   (portnum).  GetLastError()>0,  .
uint8_t rs232_ReadByte(uint8_t portnum){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	return sys_ExecuteByCodesU8(7,79);
}

//   CTS  COM-   (portnum).  GetLastError()>0,  .
uint8_t rs232_ReadCTS(uint8_t portnum){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	return sys_ExecuteByCodesU8(7,80);
}

//   DSR  COM-   (portnum).  GetLastError()>0,  .
uint8_t rs232_ReadDSR(uint8_t portnum){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	return sys_ExecuteByCodesU8(7,81);
}

//   RING  COM-   (portnum).  GetLastError()>0,  .
uint8_t rs232_ReadRING(uint8_t portnum){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	return sys_ExecuteByCodesU8(7,82);
}

//   CTS,DSR  RING  COM-   (portnum).  GetLastError()>0,  .
void rs232_ReadModemStatus(uint8_t portnum,uint8_t* cts, uint8_t* dsr,uint8_t* ring){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+1)))=(uint16_t)cts;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+3)))=(uint16_t)dsr;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+5)))=(uint16_t)ring;
	sys_ExecuteByCodes(7,83);
}
//   DTR  COM-   (portnum).  GetLastError()>0,  .
void rs232_WriteDTR(uint8_t portnum,uint8_t state){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+1)))=state;
	sys_ExecuteByCodes(7,84);
}
//   RTS  COM-   (portnum).  GetLastError()>0,  .
void rs232_WriteRTS(uint8_t portnum,uint8_t state){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+1)))=state;
	sys_ExecuteByCodes(7,85);
}

//  ,      .   ,   1.
uint8_t rs232_TestPort(uint8_t portnum){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	return sys_ExecuteByCodesU8(7,86);
}

//  .
void rs232_PurgeTX(uint8_t portnum){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	sys_ExecuteByCodes(7,87);
}

//  .
void rs232_PurgeRX(uint8_t portnum){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	sys_ExecuteByCodes(7,88);
}

//    .
void rs232_PurgeRXTX(uint8_t portnum){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	sys_ExecuteByCodes(7,89);
}

//     
uint16_t rs232_GetNumRX(uint8_t portnum){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	return sys_ExecuteByCodesU16(7,90);
}

//     
uint16_t rs232_GetNumTX(uint8_t portnum){
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR)))=portnum;
	return sys_ExecuteByCodesU16(7,91);
}



