/*
  Nanonyam
  Copyright (C) 2020 Nanoslavic
  www.nanoslavic.ru/nanonyam
  www.nanoslavic.com/nanonyam
  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
*/ 

//System and interrupts functions. Some functions require administrative privileges.

#ifndef __nanonyam_system_lib
#define __nanonyam_system_lib

//Special
#define GetLastError()		(*((volatile uint8_t*)0x0047))
#define SetLastError(err)	((*((volatile uint8_t*)0x0047))=err)

void Donates(void);//    ,      www.nanoslavic.com/donates
void sys_Nanonyam(void);
void sys_Work(void);
void sys_Delay(uint32_t t);

typedef struct {
	uint16_t SysType;//  
	uint16_t Version;//  
	uint16_t Unused1;// 
	uint16_t Unused2;// 
} sys_Info_t;
void sys_GetInfo(sys_Info_t* info);

void sys_Beep(uint16_t F, uint16_t T);

uint8_t sys_MessageBox(const char* title,const char* message,uint8_t buttons);

void sys_ProgramExit(void);
void sys_SetAutoExit(uint8_t autoexit);

void sys_RunComLine(const char* text,uint8_t show);

void sys_CursorGetXY(int16_t* mX,int16_t* mY);
void sys_CursorShow(void);
void sys_CursorHide(void);

//Timers, Date, Time
uint32_t sys_GetCount(void);
void sys_GetCountShift(uint32_t* count,uint32_t shift);
uint8_t sys_CompareCount(uint32_t* count);

typedef struct {
	uint16_t Year;
	uint8_t Month;
	uint8_t Day;
	uint8_t DayOfWeek;
	uint8_t Hours;
	uint8_t Minutes;
	uint8_t Seconds;
	uint16_t Milliseconds;
} sys_DateTime_t;

typedef struct {
	uint16_t Year;
	uint8_t Month;
	uint8_t Day;
	uint8_t DayOfWeek;//=0 =1 =2 =3 =4 =5 =6
} sys_Date_t;

typedef struct {
	uint8_t Hours;
	uint8_t Minutes;
	uint8_t Seconds;
} sys_Time_t;

void sys_GetDateTime(sys_DateTime_t *t);
void sys_SetDateTime(sys_DateTime_t *t);
void sys_GetTime(sys_Time_t *t);
void sys_GetDate(sys_Date_t *t);
void sys_GetDateTimeUTC(sys_DateTime_t *t);
void sys_GetTimeUTC(sys_Time_t *t);
void sys_GetDateUTC(sys_Date_t *t);

//Windows
void sys_WindowShow(void);
void sys_WindowHide(void);
void sys_WindowSetText(const char* text);
void sys_WindowSetNormal(void);
void sys_WindowSetMin(void);
uint8_t sys_WindowGetMin(void);
void sys_WindowFlash(void);
void sys_WindowSetZ(uint8_t z);
void sys_WindowSetXY(int16_t x, int16_t y);
void sys_WindowGetXY(int16_t *x, int16_t *y);
void sys_WindowGetSize(int16_t *width, int16_t *height);
void sys_WindowSetToCenter(void);
void sys_GetMonitorSize(int16_t *width, int16_t *height);
uint8_t sys_WindowGetFocus(void);

//Interrrupts
#define		IntParamX		(*((volatile int16_t*)48))
#define		IntParamY		(*((volatile int16_t*)50))
typedef void (*InterruptCall_t)(void);

uint8_t sys_InterruptLink(uint16_t int_type,InterruptCall_t callback);
uint8_t sys_InterruptOn(uint16_t int_type);
uint8_t sys_InterruptOff(uint16_t int_type);
uint8_t sys_InterruptGetState(uint16_t int_type);
void sys_InterruptCollectOn(void);
void sys_InterruptCollectOff(void);
void sys_InterruptWorkOn(void);
void sys_InterruptWorkOff(void);
uint16_t sys_InterruptGetNumber(void);
void sys_InterruptClear(void);
void sys_SetTimerInterval(uint8_t timerNum, uint32_t interval);

//Interrupt names and codes

/* INT_ProgramClose     */
#define INT_ProgramClose		0

/* INT_MouseLB_Down ,      
 *     IntParamX  IntParamY (   ).*/
#define INT_MouseLB_Down		1

/* INT_MouseLB_Up ,      
 *     IntParamX  IntParamY (   ).*/
#define INT_MouseLB_Up			2

/* INT_MouseLB_DBLCLK ,       
 *     IntParamX  IntParamY (   ).*/
#define INT_MouseLB_DBLCLK		3

/* INT_MouseMove ,   .
 *     IntParamX  IntParamY (   ).*/
#define INT_MouseMove			4


/* INT_Timer ,   -  ( 8 ).*/
#define INT_Timer0				5
#define INT_Timer1				6
#define INT_Timer2				7
#define INT_Timer3				8
#define INT_Timer4				9
#define INT_Timer5				10
#define INT_Timer6				11
#define INT_Timer7				12



#endif /*__nanonyam_system_lib*/


