/*
  Nanonyam
  Copyright (C) 2020 Nanoslavic
  www.nanoslavic.ru/nanonyam
  www.nanoslavic.com/nanonyam
  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
*/ 

#include <stdio.h>
#include "nanonyam.h"
#include "nn_system_lib.h"
#include "nn_text_lib.h"

//  (text)
inline void str_Clear(char* text){
text[0]=0;
}

//   (text)
uint16_t str_Size(const char* text){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	return sys_ExecuteByCodesU16(5,1);
}

//   (load)   (text)
uint16_t str_Load(char* text,const char* load){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=(uint16_t)load;
	return sys_ExecuteByCodesU16(5,2);
}

//  (load)    (text).   .
uint16_t str_LoadToEnd(char* text,const char* load){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=(uint16_t)load;
	return sys_ExecuteByCodesU16(5,3);
}

//  (from_str)   (to_str).   .
uint16_t str_Copy(char* to_str,const char* from_str){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)to_str;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=(uint16_t)from_str;
	return sys_ExecuteByCodesU16(5,4);
}


//     
void str_ToLower(char* text){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	sys_ExecuteByCodes(5,30);
}

//     
void str_ToUpper(char* text){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	sys_ExecuteByCodes(5,31);
}


//   (text)  (c)   .    ,  0xFFFF
uint16_t str_FindChar(char* text,char c){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+2)))=(uint8_t)c;
	return sys_ExecuteByCodesU16(5,5);
}

//   (text)  (find)   .    ,  0xFFFF
uint16_t str_FindText(char* text,const char* find){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=(uint16_t)find;
	return sys_ExecuteByCodesU16(5,6);
}

//   (text)  (c)      (cr).   .
uint16_t str_ReplaceChar(char* text,char c,char cr){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+2)))=(uint8_t)c;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+3)))=(uint8_t)cr;
	return sys_ExecuteByCodesU16(5,7);
}

//   (text)  (c)   .   .
uint16_t str_DeleteChar(char* text,char c){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+2)))=(uint8_t)c;
	return sys_ExecuteByCodesU16(5,10);
}

//   (text) (n)    (n0).    .
uint16_t str_DeleteN(char* text,uint16_t n0,uint16_t n){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=n0;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+4)))=n;
	return sys_ExecuteByCodesU16(5,8);
}

//   (text)  (c)   (n).    .
uint16_t str_InsertCharN(char* text,uint16_t n, char c){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=n;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+4)))=(uint8_t)c;
	return sys_ExecuteByCodesU16(5,9);
}

//  (text):  => 
void str_Reverse(char* text){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	sys_ExecuteByCodes(5,11);
}

//  (text)  ,      (c).  (c) .
void str_OnlyAfterLast(char* text,char c){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile char*)(START_SYSTEM_PARAM_ADDR+2)))=c;
	sys_ExecuteByCodes(5,20);
}

//  (text)  ,      (c).  (c) .
void str_OnlyAfter(char* text,char c){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile char*)(START_SYSTEM_PARAM_ADDR+2)))=c;
	sys_ExecuteByCodes(5,21);
}

//  (text)  ,      (c).  (c) .
void str_OnlyBeforeLast(char* text,char c){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile char*)(START_SYSTEM_PARAM_ADDR+2)))=c;
	sys_ExecuteByCodes(5,22);
}

//  (text)  ,      (c).  (c) .
void str_OnlyBefore(char* text,char c){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile char*)(START_SYSTEM_PARAM_ADDR+2)))=c;
	sys_ExecuteByCodes(5,23);
}

//  (text)  ,      (c0)     (c1).
// (c0)  (c1) .
void str_OnlyBetween(char* text,char c0,char c1){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile char*)(START_SYSTEM_PARAM_ADDR+2)))=c0;
	(*((volatile char*)(START_SYSTEM_PARAM_ADDR+3)))=c1;
	sys_ExecuteByCodes(5,24);
}

//  (text)  ,      (c0)     (c1).
// (c0)  (c1) .
void str_OnlyBeforeAndAfter(char* text,char c0,char c1){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile char*)(START_SYSTEM_PARAM_ADDR+2)))=c0;
	(*((volatile char*)(START_SYSTEM_PARAM_ADDR+3)))=c1;
	sys_ExecuteByCodes(5,25);
}

//  (text)     , :
//s="C:\Program files\program\text.txt" >>> s="text.txt"
void str_OnlyFileNameExt(char* path){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)path;
	sys_ExecuteByCodes(5,26);
}

//  (text)     , :
//s="C:\Program files\program\text.txt" >>> s="text"
void str_OnlyFileName(char* path){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)path;
	sys_ExecuteByCodes(5,27);
}

//  (text)   , :
//s="C:\Program files\program\text.txt" >>> s="txt"
void str_OnlyFileExt(char* path){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)path;
	sys_ExecuteByCodes(5,28);
}

//  (text)          .
//: s="C:\Program files\program\text.txt" >>> s="C:\Program files\program"
void str_OnlyFileDir(char* path){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)path;
	sys_ExecuteByCodes(5,29);
}

//  (numeric)  ,      (text).
// (z)   : numeric=123, z=0 => "123"; z=5 => "00123"
//    (text).
uint16_t str_U16ToText(char* text,uint8_t z,uint16_t numeric){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+2)))=z;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+3)))=numeric;
	return sys_ExecuteByCodesU16(5,12);
}

//  (numeric)  ,      (text).
// (z)   : numeric=123, z=0 => "123"; z=5 => "00123"
//    (text).
uint16_t str_I16ToText(char* text,uint8_t z,int16_t numeric){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+2)))=z;
	(*((volatile int16_t*)(START_SYSTEM_PARAM_ADDR+3)))=numeric;
	return sys_ExecuteByCodesU16(5,19);
}

//  (numeric)  ,      (text).
// (z)   : numeric=123, z=0 => "123"; z=5 => "00123"
//    (text).
uint16_t str_U32ToText(char* text,uint8_t z,uint32_t numeric){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+2)))=z;
	(*((volatile uint32_t*)(START_SYSTEM_PARAM_ADDR+3)))=numeric;
	return sys_ExecuteByCodesU16(5,13);
}

//  (numeric)  ,      (text).
// (z)   : numeric=123, z=0 => "123"; z=5 => "00123"
//    (text).
uint16_t str_I32ToText(char* text,uint8_t z,int32_t numeric){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+2)))=z;
	(*((volatile int32_t*)(START_SYSTEM_PARAM_ADDR+3)))=numeric;
	return sys_ExecuteByCodesU16(5,17);
}

//  (numeric)  ,      (text).
// (z)   : numeric=123, z=0 => "123"; z=5 => "00123"
//    (text).
uint16_t str_U64ToText(char* text,uint8_t z,uint64_t numeric){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+2)))=z;
	(*((volatile uint64_t*)(START_SYSTEM_PARAM_ADDR+3)))=numeric;
	return sys_ExecuteByCodesU16(5,14);
}

//  (numeric)  ,      (text).
// (z)   : numeric=123, z=0 => "123"; z=5 => "00123"
//    (text).
uint16_t str_I64ToText(char* text,uint8_t z,int64_t numeric){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+2)))=z;
	(*((volatile int64_t*)(START_SYSTEM_PARAM_ADDR+3)))=numeric;
	return sys_ExecuteByCodesU16(5,18);
}

//  (x)  ,      (text).
// (n)   .     (text).
uint16_t str_FloatToText(char* text,uint8_t n,float x){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+2)))=n;
	(*((volatile float*)(START_SYSTEM_PARAM_ADDR+3)))=x;
	return sys_ExecuteByCodesU16(5,15);
}



//  (numeric)  ,      (text).
//       2 .
//    (text).
uint16_t str_U8ToHex(char* text,uint8_t numeric){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint8_t*)(START_SYSTEM_PARAM_ADDR+2)))=numeric;
	return sys_ExecuteByCodesU16(5,32);
}

//  (numeric)  ,      (text).
//       4 .
//    (text).
uint16_t str_U16ToHex(char* text,uint16_t numeric){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR+2)))=numeric;
	return sys_ExecuteByCodesU16(5,33);
}

//  (numeric)  ,      (text).
//       8 .
//    (text).
uint16_t str_U32ToHex(char* text,uint32_t numeric){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint32_t*)(START_SYSTEM_PARAM_ADDR+2)))=numeric;
	return sys_ExecuteByCodesU16(5,34);
}

//  (numeric)  ,      (text).
//       16 .
//    (text).
uint16_t str_U64ToHex(char* text,uint64_t numeric){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	(*((volatile uint64_t*)(START_SYSTEM_PARAM_ADDR+2)))=numeric;
	return sys_ExecuteByCodesU16(5,35);
}


int32_t str_ToI32(char* text){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	return (int32_t)sys_ExecuteByCodesU32(5,36);
}

int64_t str_ToI64(char* text){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	return (int64_t)sys_ExecuteByCodesU64(5,37);
}

float str_ToFloat(char* text){
	(*((volatile uint16_t*)(START_SYSTEM_PARAM_ADDR)))=(uint16_t)text;
	return sys_ExecuteByCodesFLT(5,38);
}
